/* IRCfs - IRC FileServ for *nix.
 * Copyright (C) 2002  Nick 'Zaf' Clifford <zaf@nrc.co.nz>
 * For licensing details, refer to the LICENSE file in the source 
 * code directory.
 */
#ifndef __include_str_hash_h__
#define __include_str_hash_h__

#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include "runtime.h"

struct str_hash_table;

typedef int (*str_hash_callback_t)(struct str_hash_table *, const char *name,
		void *contents, void *appdata); 

struct str_hash_table *str_hash_create(POOL parent);
struct str_hash_table *str_hash_create_n(POOL parent, size_t buckets);
/* destroys a hash table.
 * free's any memory that *IT* allocated. eg
 * contents of nodes are *NOT* destroyed.
 * To do a total clean up, call hash_foreach, and free each
 * node yourself, then call hash_destroy
 */
int str_hash_destroy(struct str_hash_table *ht);

int str_hash_resize(struct str_hash_table *ht, size_t buckets);

int str_hash_foreach(struct str_hash_table *ht, str_hash_callback_t callback,
		void *appdata);

int str_hash_add(struct str_hash_table *ht, const char *name, void *contents);
int str_hash_set(struct str_hash_table *ht, const char *name, void *contents,
		void **old);

/* Returns the number of entries in the hash table */
int str_hash_count(struct str_hash_table *ht);

/* hash_del(ht,name)
 * Deletes node name from hash table.
 * Returns contents of node if found and deleted.
 * Otherwise returns NULL
 */
void *str_hash_del(struct str_hash_table *ht, const char *name);

void *str_hash_get(struct str_hash_table *ht, const char *name);
void *str_hash_get_n(struct str_hash_table *ht, int num);

#endif
